"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NamespaceExportConverter = exports.ExportSpecifierConverter = exports.ExportConverter = void 0;
const ts = require("typescript");
const assert = require("assert");
const index_1 = require("../../models/index");
const components_1 = require("../components");
const reference_1 = require("../factories/reference");
// Either a default export or export=
let ExportConverter = class ExportConverter extends components_1.ConverterNodeComponent {
    constructor() {
        super(...arguments);
        this.supports = [ts.SyntaxKind.ExportAssignment];
    }
    convert(context, node) {
        var _a, _b;
        var _c;
        assert(context.scope instanceof index_1.ContainerReflection);
        const name = node.isExportEquals ? "export =" : "default";
        // We might not have a symbol if someone does `export default 1`
        const expressionSymbol = context.getSymbolAtLocation(node.expression);
        if (expressionSymbol) {
            const reflection = reference_1.createReferenceOrDeclarationReflection(context, expressionSymbol, context.resolveAliasedSymbol(expressionSymbol));
            if (reflection) {
                reflection.name = name;
            }
            return reflection;
        }
        else {
            // We can't use createDeclaration because it expects a ts.Declaration, which we don't have.
            const reflection = new index_1.DeclarationReflection(name, index_1.ReflectionKind.Variable, context.scope);
            const exportSymbol = (_a = node.symbol) !== null && _a !== void 0 ? _a : context.expectSymbolAtLocation(node);
            reflection.type = this.owner.convertType(context, context.checker.getTypeOfSymbolAtLocation(exportSymbol, node));
            (_b = (_c = context.scope).children) !== null && _b !== void 0 ? _b : (_c.children = []);
            context.scope.children.push(reflection);
            return reflection;
        }
    }
};
ExportConverter = __decorate([
    components_1.Component({ name: "node:export-assignment" })
], ExportConverter);
exports.ExportConverter = ExportConverter;
let ExportSpecifierConverter = class ExportSpecifierConverter extends components_1.ConverterNodeComponent {
    constructor() {
        super(...arguments);
        this.supports = [ts.SyntaxKind.ExportSpecifier];
    }
    convert(context, node) {
        var _a;
        const source = context.expectSymbolAtLocation(node.name);
        const target = context.resolveAliasedSymbol(context.expectSymbolAtLocation((_a = node.propertyName) !== null && _a !== void 0 ? _a : node.name));
        return reference_1.createReferenceOrDeclarationReflection(context, source, target);
    }
};
ExportSpecifierConverter = __decorate([
    components_1.Component({ name: "node:export-specifier" })
], ExportSpecifierConverter);
exports.ExportSpecifierConverter = ExportSpecifierConverter;
let NamespaceExportConverter = class NamespaceExportConverter extends components_1.ConverterNodeComponent {
    constructor() {
        super(...arguments);
        this.supports = [ts.SyntaxKind.NamespaceExport];
    }
    convert(context, node) {
        const source = context.expectSymbolAtLocation(node.name);
        const target = context.resolveAliasedSymbol(source);
        return reference_1.createReferenceOrDeclarationReflection(context, source, target);
    }
};
NamespaceExportConverter = __decorate([
    components_1.Component({ name: "node:namespace-export" })
], NamespaceExportConverter);
exports.NamespaceExportConverter = NamespaceExportConverter;
//# sourceMappingURL=export.js.map