"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MappedType = void 0;
const abstract_1 = require("./abstract");
/**
 * Represents a mapped type.
 *
 * ```ts
 * { -readonly [K in keyof U]?: Foo }
 * ```
 */
class MappedType extends abstract_1.Type {
    constructor(parameter, parameterType, templateType, readonlyModifier, optionalModifier) {
        super();
        this.parameter = parameter;
        this.parameterType = parameterType;
        this.templateType = templateType;
        this.readonlyModifier = readonlyModifier;
        this.optionalModifier = optionalModifier;
        this.type = "mapped";
    }
    clone() {
        return new MappedType(this.parameter, this.parameterType.clone(), this.templateType.clone(), this.readonlyModifier, this.optionalModifier);
    }
    equals(other) {
        return (other instanceof MappedType &&
            other.parameter == this.parameter &&
            other.parameterType.equals(this.parameterType) &&
            other.templateType.equals(this.templateType) &&
            other.readonlyModifier === this.readonlyModifier &&
            other.optionalModifier === this.optionalModifier);
    }
    toString() {
        var _a, _b;
        const read = {
            "+": "readonly",
            "-": "-readonly",
            "": "",
        }[(_a = this.readonlyModifier) !== null && _a !== void 0 ? _a : ""];
        const opt = {
            "+": "?",
            "-": "-?",
            "": "",
        }[(_b = this.optionalModifier) !== null && _b !== void 0 ? _b : ""];
        return `{ ${read}[${this.parameter} in ${this.parameterType}]${opt}: ${this.templateType}}`;
    }
}
exports.MappedType = MappedType;
//# sourceMappingURL=mapped.js.map