"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReflectionTypeSerializer = void 0;
const models_1 = require("../../../models");
const components_1 = require("../../components");
class ReflectionTypeSerializer extends components_1.TypeSerializerComponent {
    constructor() {
        super(...arguments);
        this.visited = new Set();
    }
    supports(t) {
        return t instanceof models_1.ReflectionType;
    }
    toObject(reference, obj) {
        const result = {
            ...obj,
        };
        // Because `DeclarationReflection` has reference to multiple types objectifying a declaration
        // on a type might fall into a loop trap (cyclic dependency).
        // The TypeDoc code does not apply logic that can create this scenario but a 3rd party plugin
        // might do that unintentionally so a protection is in place.
        // TODO: Should this protection really be here? It seems like it might make more sense to
        // do this check in the DeclarationReflection serializer... if it should even be checked. In the
        // old `.toObject` code, it wasn't checked.
        if (this.visited.has(reference.declaration)) {
            // if we're here it means that the reference type is rendered for the 2nd time
            // by the declaration it is referencing, we will render a pointer-only declaration.
            result.declaration = { id: reference.declaration.id };
        }
        else {
            // mark this declaration to trap a loop
            this.visited.add(reference.declaration);
            // objectify the declaration
            result.declaration = this.owner.toObject(reference.declaration);
        }
        // no more declaration rendering, remove marker.
        this.visited.delete(reference.declaration);
        return result;
    }
}
exports.ReflectionTypeSerializer = ReflectionTypeSerializer;
//# sourceMappingURL=reflection.js.map